"""Base class for an output Device (complete documented)

A Device is needed to render plots on. This class can't do that, but is a base
class for such Devices.
e.g Kplot.pgplot.Device is such a Device, it inherits this class and implements
the methods needed.

You can use this as a reference for all Devices, all Devices should behave
like this Device describes.

for developers, look at Kplot.DeviceAbstract if you want to make a new Device

Kplot has subpackages with implementations for this Device class. like
Kplot.pgplot.Device, but use of any other package will be the same as
this description.

a Device's constructor has at least 2 arguments, size, and units(the units
size is expressed in)

dev = Kplot.pgplot.Device(size=(10, 10), units="cm")
opens a PGPLOT window with 10x10 centimeter

so this should also work
dev = Kplot.<futurepackage>.Device(size=(10, 10), units="cm")


Devices can have additional information, see the Device's docstring
for details about the Device.
	help(Kplot.pgplot.Device)

"""
from matrix import Matrix
from tools import abstract



class CapsException(Exception):
	pass

class Device:
	"""A Device represents a target to render the plots on.

	Device can do very primitive plotting operations. It can plot lines,
	polygons text, and images. These basic operations should be
	enough to construct more complex objects, like histograms axes, grids.

	You can't make use this class to directly, it's an abstract class. Use
	Kplot.pgplot.Device for example.

	A Device can have an number of Panels as children.

	A Device has a minimum of these these linestyles, fillstyles and fonts:

	linestyle:
		- "noline"
		- "normal"
		- "dashed"
		- "dotted"

	fillstyle:
		- "nofill"
		- "solid"
		- "hatched"
		- "crosshatched"
	font:
		- "normal"   -A sans serif like font, like Helvetica, Arial, Verdana
		- "roman"    -A serif like font, like Times-Roman,
		- "italic"   -An italic font, like Times-Italic

	Use getLinstyles, getFillstyles, getFonts to get a list of all styles
	supported by the device.

	linewidth, fontsize, weight are expressed in units.
	The default units are "mm", this is used when the units argument in one
	of the plot methods is None.

	list of available units:
		- "normalised" (normalised device coordinates, (0,0)-(1,1) range
		- "inch"
		- "points"     (a point is 1/72 of an inch)
		- "mm"         (millimeter)
		- "cm"         (centimeter)

	Coordinates are all in world coordinates, they undergo several linear
	transformations to map them to device specific coordinates.


	world coordinates
		|
		| (linear transformation)
		v
	viewport coordinates (normalised)
		|
		| (linear transformation)
		v
	normalised device coordinates (normalised)
		|
		| (linear transformation)
		v
	device dependant coordinates

	These three transformations are implemented using matrices.
	totalmatrix = devicematrix * viewportmatrix * worldmatrix

	The devicematrix is not important to you, but the worldmatrix is.

	"""

	def __init__(self):
		self.children = []

	def output(self, other=None):
		"""Ouputs all Panels and PlotObjects to the Device
		First the device is cleared, then each Panel is drawn
		"""
		abstract()

	def plotPolyline(self, x, y, linestyle, linewidth, color, units):
		"""Plots a polyline

		Arguments:
		x -- sequence of x values
		y -- sequence of y values
		linestyle -- see above
		linewidth -- see above
		color -- a Kplot.Color object, or a string representing a color,
			like Color.green, Color(0,1,0) or "green"
		units -- When None, the default units are used("mm")
		"""
		abstract()

	def plotPolygon(self, x, y, fillstyle, linestyle, linewidth, color, units):
		"""Plots a polygon using odd-even rule

		Arguments:
		x -- sequence of x values
		y -- sequence of y values
		fillstyle -- see above
		linestyle -- see above
		linewidth -- see above
		color
		units -- When None, the default units are used("mm")
		"""
		abstract()

	def plotLines(self, x, y, linestyle, linewidth, color, units):
		"""Plots multiple lines"

		Arguments:
		x -- sequence of x values, should be even of length
		y -- sequence of y values, should be of equal length of x
		linestyle -- see above
		linewidth -- see above
		color -- see anove
		units -- When None, the default units are used("mm")
		"""
		abstract()

	def plotCircle(self, center, radius, fillstyle, linestyle, linewidth, color, units):
		"""Plots a single circle

		The circle is plotted in world coordinates, and might not be a true circle when
		the aspect ratio of the world coordinates is not 1.

		The plotCircle function should be replaced with plotPie/plotArc/plotChord,
		PGPLOT can't draw these, but could be emulated.
		
		Arguments:
		center -- a (x, y) tuple (or any sequence of length 2)
		radius -- the radius of the circle in world coordinates
		fillstyle -- see above
		linestyle -- see above
		linewidth -- see above
		color -- see anove
		units -- When None, the default units are used("mm")
		"""
		abstract()
	
	#def plotPie(self, p1, p2, startangle, endangle, fillstyle, linestyle, linewidth, color, units=None):
	#	abstract()

	#def plotArc(self, p1, p2, radius, startangle, endangle, linestyle, linewidth, color, units=None):
	#	abstract()

	#def plotChord(self, p1, p2, radius, startangle, endangle, linestyle, linewidth, color, units=None):
	#	abstract()

	def plotText(self, text, location, font, fontsize, weight, color, bgcolor, textangle, valign, halign, units):
		"""Plots text
		Escape sequences are Device specific, check the device you are using
		for options. Newline characters(\\n) also don't work.

		Arguments:
		text -- a single line text string
		location -- a 2d point(sequence of length 2, (10,12) or [13,14])
		font -- see above
		fontsize -- height of the font, expressed in units
		weight -- thickness of the outlines, expressed in units
		color
		bgcolor -- When this is None, the background is transparant
		textangle -- the angle in radians of the baseline, counterclockwise
		valign -- vertical alignment, "top", "center" or "bottom"
		halign -- horizontal alignment, "left", "center" or "right"
		units -- When None, the default units are used("mm")
		"""
		abstract()

	def plotImage(self, data, colormap, datamin, datamax, function):
		"""Plot a 2 dimensional sequence as an image

		The color plotted depends on the colormap, datamin, datamax, and
		function.

		The data in the [datamin, datamax] rage gets normalised to the
		[0,1] range. Everything outside that range gets clipped to datamin
		and datamax. This value is used as argument to function, which
		may return a value outside the [0,1] range, but will be clipped to
		it.
		The result will be used as argument for the colormap, and returns
		the color used for the image pixel.

		data: [-------|datamin|-----------------------------|datamax|---------------]
		                 /                                      \\
		normalised:    [0---------------------------------------1]
		                                 |
		                                 V
		                      value = function(value)
		                                 |
		                                 V
		                      value = clip(value, 0, 1)
		                                 |
		                                 V
		                      color = colormap(value)


		Even when you want to plot 1 pixel, use a 2 dimensional sequence
		like [[1]].
		The data is in [row][column] format.
		[[1,2,3,4,5], [5,4,3,2,1]] will be displayed as 2 rows with 5 pixels.

		Each pixel from data is plotted at the center of a world coordinate,
		starting at (1,1). And each pixel is 1x1 world coordinates.
		To plot an image you with with=10 and height=20 you normally
		want your world coordinates to be p1=(0.5, 0.5), p2=(20.5, 20.5)

		The datamin and datamax range of the image will be mapped to the
		colormap. This can be used to view a small range in the image.

		The function can be used to transform the value used for the colormap,
		like take the inverse:
			def inverse(value)
				return 1.0 - value
			image = Kplot.Image([[1,2,3],[2,3,4]], function=inverse)
		or square
			image = Kplot.Image([[1,2,3],[2,3,4]], function=lambda x: x ** 2)
		or the square root using the math.sqrt function
			import math
			image = Kplot.Image([[1,2,3],[2,3,4]], function=math.sqrt)

		see Kplot.colormap for some details about colormaps, and some
		predefined colormaps.
		
		Arguments:
		data -- a 2 dimensional sequence
		colormap -- a callable that takes 1 arguments in the [0,1] range and
			returns a Color object
		datamin -- the value that gets mapped to the color of colormap(0.0)
		datamax -- the value that gets mapped to the color of colormap(1.0)
		function -- a callable that may transform the value used for the colormap

		"""
		abstract()

	def pushViewportMatrix(self, matrix):
		"""Multiplies the matrix with the top of the viewport matrix stack

		The result is pushed on the viewport matrix stack, a popViewportMatrix
		returns the stack in it's original state
		pushViewportMatrix and popViewportMatrix should always be used in pairs

		The top viewport matrix should transform viewport coordinates to normalised
		device coordinates.
		"""
		abstract()

	def popViewportMatrix(self):
		"""Removes the top of the viewport stack"""
		abstract()

	def getViewportMatrix(self):
		"""Return the top of the viewport matrix stack"""
		abstract()

	def getLastViewportMatrix(self):
		"""Return the matrix that was last pushed onto the viewport matrix stack"""
		abstract()

	def pushWorldMatrix(self):
		"""Multiplies the matrix with the top of the world matrix stack

		The result is pushed on the world matrix stack, a popWorldMatrix
		returns the stack in it's original state
		pushWorldMatrix and pushWorldMatrix should always be used in pairs

		The top world matrix should transform world coordinates to (normalised)
		viewport coordinates
		"""
		abstract()

	def popWorldMatrix(self):
		"""Removed the top of the world matrix stack"""
		abstract()

	def getWorldMatrix(self):
		"""Return the top of the world matrix stack"""
		abstract()

	def getLastWorldMatrix(self):
		"""Return the matrix that was last pushed on the world matrix stack"""
		abstract()

	def getFonts(self):
		"""Return a sequence of all fonts(strings)"""
		abstract()

	def getLinestyles(self):
		"""Return a sequence of all linestyles(strings)"""
		abstract()

	def getFillstyles(self):
		"""Return a sequence of all fillstyles(strings)"""
		abstract()

	def setClipping(self, enabled):
		"""Disables or enables all clipping

		Arguments:
		enabled -- boolean, if false, clipping is disabled
		"""
		abstract()

	def getClipping(self):
		abstract()

	def createPanel(self, p1=(0.1,0.1), p2=(0.9, 0.9), parent=None):
		"""Return a Panel that is a child of this Device

		Arguments:
		p1 -- the lower right part of the viewport
		p2 -- the upper right part of the viewport

		Optional Arguments:
		parent -- This is used internally, when a child of a Panel is created
		"""
		abstract()

	def printMembers(self, indent=0):
		print "    "*indent, self.__module__ +"." +self.__class__.__name__

		for child in self.children:
			child.printMembers(indent+1)

	def setDefaultUnits(self, units):
		"""Changes the default units
		This is not recommended, because the default values for PlotObjects
		are set with "mm"(millimeters) in mind
		"""
		abstract()

	def getDimension(self, units=None):
		"""Gets the dimension of the Device in units"""
		abstract()

	def getUnitMatrix(self, units=None):
		"""Returns a matrix that transforms coordinates expressed i units to normalised"""
		abstract()

	def getPos(self, clip=1):
		abstract()

	def changeSize(self, size, units=None):
		"""Changes the size of the Device, size expressed in units"""
		abstract()

	def open(self):
		"""Opens the Device
		When a Device is created it always starts in open state. But a Device
		may be closed and opened as many times as requried. As long as they are
		called in pairs.
		"""
		abstract()

	def close(self, prompt=True):
		"""Closes the device, drawing in this state results in undefined behaviour.
		Argument:
			prompt -- If promts is True, close might prompt before closing,
				some Devices, which output to file may or should ignore this,
				and never prompt.

		"""
		abstract()

	def setClipping(self, enable):
		"""Sets clipping of primitives agains viewport on/off"""
		abstract()

	def getClipping(self):
		"""Returns True if clipping is on"""
		abstract()

	def hardcopy(self, filename=None, filetype=None):
		"""Creates a hardcopy of the current plot.
		
		The Kplot.createHardcopyDevice function is used as device for making a
		hardcopy. This depends on the installed software or the KPLOT_HARDCOPY_DEVICE
		env var. See Kplot.createHardcopyDevice for more detail.
		
		Arguments:
		filename -- filename without suffix/extensions, "test" instead of "test.ps"
					The device knows which extension to add to the filename.
					Default filename is "Kplot"
		filetype -- output file type. set to 'gif' to get gif file, set to 'ps' to get postscript.
		                        If not set the default KPLOT_HARDCOPY_DEVICE is used.
					

		"""
		pass

	def setFgColor(self, color):
		"""Sets the foreground color"""
		abstract()

	def setBgColor(self, color):
		"""Sets the background color"""
		abstract()
		
