"""Used as base class for new Devices, only of interest for developers"""
from device import Device
from matrix import Matrix
from panel import Panel
from Kplot.tools import abstract
import sys

class DeviceAbstract(Device):
	"""Used as a base class for Devices

	Does some basic things so not every subclass of Device has to
	implement it.

	As minimum, a Device has to implement these methods:
	getDeviceMatrix
	_plotPolyline: x,y coordinates are transformed
	_plotPolygon:  using world, viewport and
	_plotText:     device matrix(getDeviceMatrix)
	plotImage
	clear
	changeSize
	open
	close
	getLinestyles
	getFillstyles
	getFonts

	And a constructor with 2 arguments:
		size(150, 150), and units=None
	so that the default size, is 150x150mm.
	See Kplot.dummy.Device as an example
	"""
	def __init__(self):
		Device.__init__(self)
		self.units = "mm"
		self.worldstack = [Matrix()]
		self.viewportstack = [Matrix()]
		self.worldmatrices = []
		self.viewportmatrices = []
		self.dimensions = {}
		self.dimensions["normalised"] = (1,1)
		self.clipping = True
		self.doingouput = False
		self.autoreplot = hasattr(sys, "ps1")

	def getTotalMatrix(self):
		worldmatrix = self.getWorldMatrix()
		viewportmatrix = self.getViewportMatrix()
		devicematrix = self.getDeviceMatrix()
		matrix = devicematrix * viewportmatrix * worldmatrix
		return matrix

	def _plotPolyline(self, x, y, linestyle, linewidth, color, units=None):
		abstract()

	def plotPolyline(self, x, y, linestyle, linewidth, color, units=None):
		matrix = self.getTotalMatrix()
		x, y = matrix.mulXY(x, y)
		self._plotPolyline(x,y, linestyle=linestyle, linewidth=linewidth, color=color, units=units)

	def _plotPolygon(self, x, y, fillstyle, linestyle, linewidth, color, units=None):
		abstract()

	def plotPolygon(self, x, y, fillstyle, linestyle, linewidth, color, units=None):
		matrix = self.getTotalMatrix()
		x, y = matrix.mulXY(x, y)
		self._plotPolygon(x,y, fillstyle=fillstyle, linestyle=linestyle, linewidth=linewidth, color=color, units=units)

	#def _plotArc(self, p1, p2, fillstyle, linestyle, linewidth, color, startangle, anglelength, units=None):
	#	abstract()

	#def plotArc(self, p1, p2, fillstyle, linestyle, linewidth, color, startangle, anglelength, units=None):
	#	matrix = self.getTotalMatrix()
	#	p1 = matrix * p1
	#	p2 = matrix * p2
	#	self._plotArc(p1, p2, fillstyle=fillstyle, linestyle=linestyle, linewidth=linewidth, color=color,
	#		startangle=startangle, anglelength=anglelength, units=units)

	def _plotText(self, text, location, font, weight, fontsize, color, bgcolor, textangle, valign, halign, units=None):
		abstract()

	def plotText(self, text, location, font,weight, fontsize, color, bgcolor, textangle, valign, halign, units=None):
		matrix = self.getTotalMatrix()
		location = matrix * location
		self._plotText(text, location, font=font, weight=weight, fontsize=fontsize, color=color, bgcolor=bgcolor,
			textangle=textangle, valign=valign, halign=halign, units=units)


	def pushViewportMatrix(self, matrix):
		self.viewportmatrices.append(matrix)
		self.viewportstack.append(self.viewportstack[-1] * matrix)

	def popViewportMatrix(self):
		self.viewportstack.pop()

	def pushWorldMatrix(self, matrix):
		self.worldmatrices.append(matrix)
		self.worldstack.append(self.worldstack[-1] * matrix)

	def getLastWorldMatrix(self):
		return self.worldmatrices[-1]

	def getLastViewportMatrix(self):
		return self.viewportmatrices[-1]

	def popWorldMatrix(self):
		self.worldstack.pop()

	def getDeviceMatrix(self):
		abstract()

	def getWorldMatrix(self):
		return self.worldstack[-1]

	def getViewportMatrix(self):
		return self.viewportstack[-1]

	def setClipping(self, enabled):
		self.clipping = enabled

	def getClipping(self):
		return self.clipping

	def setDefaultUnits(self, units):
		self.units = units

	def getDimension(self, units=None):
		if units == None:
			units = self.units
		return self.dimensions[units]

	def getUnitMatrix(self, units=None):
		width, height = self.getDimension(units)
		return Matrix.scaleXY(1.0/width, 1.0/height)

	def createPanel(self, p1=(0.1,0.1), p2=(0.9, 0.9), parent=None):
		if parent == None:
			parent = self
		panel = Panel(p1=p1, p2=p2, parent=parent)
		if parent == self:
			self.children.append(panel)
		return panel

	def output(self):
		self.clear()
		self.worldstack = [Matrix()]
		self.viewportstack = [Matrix()]
		self.worldmatrices = []
		self.viewportmatrices = []
		self.doingouput = True
		try:
			self.preOutput()
			for child in self.children:
				child.plot(self)
			self.postOutput();
		except:
			self.doingouput = False
			raise
		self.doingouput = False
		
	def preOutput(self):
		pass
		
	def postOutput(self):
		pass

	def __clonePanel(self, sourcepanel, parent):

		panel = parent.createPanel()
		panel.viewportmatrix = Matrix(sourcepanel.viewportmatrix)
		panel.worldmatrix = Matrix(sourcepanel.worldmatrix)
		panel.background.color = sourcepanel.background.color
		panel.background.fillstyle = sourcepanel.background.fillstyle
		panel.background.linestyle = sourcepanel.background.linestyle
		panel.clipping = sourcepanel.clipping
		panel.objects = sourcepanel.objects
		for child in sourcepanel.children:
			self.__clonePanel(child, panel)
		#p1, p2 = source.getViewport()

		#panel = target.createPanel(p1=p1, p2=p2)
		#panel.clipping = source.clipping
		#panel.worldmatrix = Matrix(source.worldmatrix)
		#panel.fillstyle = source.fillstyle
		#panel.color = source.color
		#panel.background = source.background
		#panelcopy  = panel.copy()
		#panelcopy.objects = panel.objects
		#return panelcopy




	def outputto(self, device):
		for child in self.children:
			self.__clonePanel(child, device)
		device.output()
		device.close()

	def hardcopy(self, filename=None, filetype=None, orientation="portrait"):
		import Kplot
		dev = Kplot.createHardcopyDevice(filename=filename, filetype=filetype, orientation=orientation)
		arp = self.autoreplot
		self.autoreplot = False
		self.outputto(dev)
		self.autoreplot = arp


	def changeNotify(self):
		if (not self.doingouput) and self.autoreplot:
			self.output()


	def setAutoReplot(self, enable):
		self.autoreplot = enable

	def reset(self):
		for child in self.children:
			child.parent = None
			del child
		self.children = []
		self.clear()
