"""Arrow"""
from plotobject import PlotObject
from properties import *
from autoproperty import *
import math
from Kplot import toradians
#import N

class Arrow(PlotObject):
	"""Draws a line with an arrowhead at it's end point from p1 to p2

	Input are two points
	(1st is start point, 2nd is end point, where the arrowhead will be)

	Properties:
	- p1
	- p2
	- headsize
	  (size of the head relative to the length of the arrow, between 0 and 1)
	- headangle
	  (angle of head in degrees)
	- headbarb
	  (the fraction of the triangular arrow-head that is cut away from the back.
	   0.0 gives a triangular wedge arrow-head; 1.0 gives an open >)
	- color
	- linestyle 
	- linewidth 
	- fillstyle 
	- matrix
	- units
	
	"""
	headsize = numberproperty(default = 0.2, docstring="size of the head relative to the length of the arrow, between 0 and 1")
	headangle = numberproperty(default = toradians(30), docstring="angle of head in degrees")
	headbarb = numberproperty(default = 0.5, docstring="the fraction of the triangular arrow-head that is cut away from the back. 0.0 gives a triangular wedge arrow-head; 1.0 gives an open >")
	color = colorproperty()
	linestyle = linestyleproperty()
	linewidth = linewidthproperty()
	fillstyle = fillstyleproperty()
	matrix = matrixproperty()
	p1 = pointproperty()
	p2 = pointproperty()
	units = stringproperty()

	def __init__(self, p1, p2, defaults=None, **properties):
		"""Constructor

		Arguments:
		p1 -- the start point of the arrow
		p2 -- the end point of the arrow, the head is located at this point
		"""
		PlotObject.__init__(self, defaults=defaults, properties=properties)
		self.p1 = p1
		self.p2 = p2

	def plot(self, dev):
		dev.pushWorldMatrix(self.matrix)
		worldmatrix = dev.getWorldMatrix()
		dev.pushWorldMatrix(worldmatrix.inverse())

		p1 = worldmatrix * self.p1
		p2 = worldmatrix * self.p2

		if self.headsize > 0.0:
			arrowvec = (p1-p2).scale(self.headsize)
			midpoint = p2 + arrowvec.scale(1-self.headbarb)
			rotleft = Matrix.rotate(-self.headangle)
			rotright = Matrix.rotate(self.headangle)
			scale = Matrix(rotleft)
			scale[0][0] = 1.0 / scale[0][0]
			scale[1][1] = 1.0 / scale[1][1]
			scale[1][0] = 0
			scale[0][1] = 0
			leftpoint = p2 + rotleft * scale * arrowvec
			rightpoint = p2 + rotright * scale * arrowvec
			x = [midpoint.x, leftpoint.x, p2.x, rightpoint.x]
			y = [midpoint.y, leftpoint.y, p2.y, rightpoint.y]
			dev.plotPolygon(x, y, color=self.color, linestyle=self.linestyle, linewidth=self.linewidth, fillstyle=self.fillstyle, units=self.units)
		else:
			midpoint = p2

		dev.plotPolyline([p1.x, midpoint.x], [p1.y, midpoint.y], color=self.color, linestyle=self.linestyle, linewidth=self.linewidth, units=self.units)


		dev.popWorldMatrix()
		dev.popWorldMatrix()


	def example():
		import Kplot

		print "Arrow example"
		print
		print 'An Arrow needs two points as input,'
		print 'a start point and and end point, where the head will be.'
		print 
		print 'An Arrow is basically a Line and a Polygon, '
		print 'so it has a lot of properties. Use help to get a list.'
		print
		print 'arrow2 = Kplot.Arrow((1, 0), (0, 1), color="purple")'
		print 'arrow1 = Kplot.Arrow((0, 0), (1, 1), headsize=0.1, headbarb=0.0,'
		print '                     headangle=toradians(15))'
		arrow1 = Kplot.Arrow((0, 0), (1, 1), headsize=0.1, headbarb=0.0, headangle=toradians(15))
		arrow2 = Kplot.Arrow((1, 0), (0, 1), color="purple")
		dev = Kplot.createDevice()
		panel = dev.createPanel()
		panel.add(arrow1, arrow2)
		dev.output()
		dev.close()
		print

	example = staticmethod(example)

	def test():
		import Kplot

		arrows = []
		headangle = toradians(30)
		headbarb = 0.5
		headsize = 0.5
		arrows.append(Kplot.Arrow((0, 0), (2, 1), headsize=headsize, headbarb=headbarb, headangle=headangle))
		arrows.append(Kplot.Arrow((0, 0), (2, 0), headsize=headsize, headbarb=headbarb, headangle=headangle))
		arrows.append(Kplot.Arrow((0, 0), (2,-1), headsize=headsize, headbarb=headbarb, headangle=headangle))

		arrows.append(Kplot.Arrow((0, 0), (-2, 1), headsize=headsize, headbarb=headbarb, headangle=headangle, fillstyle="nofill"))
		arrows.append(Kplot.Arrow((0, 0), (-2, 0), headsize=headsize, headbarb=0.1, headangle=headangle))
		arrows.append(Kplot.Arrow((0, 0), (-2,-1), headsize=0.1, headbarb=headbarb, headangle=headangle))

		arrows.append(Kplot.Arrow((0, 0), (0, 1), headsize=headsize, headbarb=headbarb, headangle=headangle))
		arrows.append(Kplot.Arrow((0, 0), (0,-1), headsize=headsize, headbarb=headbarb, headangle=headangle))

		dev = Kplot.createDevice()
		panel = dev.createPanel()
		panel.add(arrows)
		panel.setWorld(p1=(-2, -1), p2=(2,1))
		dev.output()
		dev.close()

	test = staticmethod(test)


