"""An axis with labels at the tickmarks, and minor tickmarks"""
from plotobject import PlotObject
from properties import *
from basic import Text
from nonbasic import Line
from math import sqrt, pi, log10, floor, atan
from Kplot import Vector
from Kplot.tools import abstract
import nonbasic
from Kplot.tools import *

def lin2log(x):
	xOffset = abs(floor(x))
	x += xOffset
	y = log10((x-int(x))*10) + int(x)
	y-=xOffset 
	return y

class _AxisLine:
	"""Used in Axis
	"""
	def __init__(self, p1, p2):
		self.p1 = p1
		self.p2 = p2

	def ytick(self, tick, log=0):
		height = self.p2.y - self.p1.y
		if height != 0.0:
			scale = (tick - self.p1.y) / height
			if scale != None and scale >= 0.0 and scale <= 1.0:
				point = self.p1 + (self.p2 - self.p1).scale(scale)
				if log:
					point.y = lin2log(point.y)
					if point.y > max([self.p1.y,self.p2.y]) or point.y < min([self.p1.y,self.p2.y]):
						point = None
				return point
			else:
				return None
		else:
			return None

	def xtick(self, tick, log=0):
		width = self.p2.x - self.p1.x
		if width != 0.0:
			scale = (tick - self.p1.x) / width
			if scale != None and scale >= 0.0 and scale <= 1.0:
				point = self.p1 + (self.p2 - self.p1).scale(scale)
				if log:
					point.x = lin2log(point.x)
					if point.x > max([self.p1.x,self.p2.x]) or point.x < min([self.p1.x,self.p2.x]):
						point = None
				return point
			else:
				return None
		else:
			return None

class LabelFormatter:
	"""Subclass this to make your own label formatter
	example:
		class MonthLabelFormatter(LabelFormatter):

			months = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"]

			def init(self, minvalue, maxvalue):
				pass

			def format(self, value):
				return self.months[value]

	"""
	def init(self, minvalue, maxvalue):
		abstract()
	def format(self, value):
		abstract()

class MonthLabelFormatter(LabelFormatter):
	"""For labeling with months
	If the tick value used is a float, e.g the values 1.4 or 1.8 will be cast to an int,
	and result in 1, the 2nd month, "Feb".
	the value is used as an index in a list, so value 0 will correspond to "Jan"
	and value 11 will correspond to month "Dec". [0-11], instead of [1-12]
	"""

	months = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"]

	def init(self, minvalue, maxvalue):
		pass

	def format(self, value):
		return self.months[int(value)]

class DefaultLabelFormatter(LabelFormatter):
	"""Formats the value using "%g" % value
	This usually results in nice labels
	"""
	def init(self, minvalue, maxvalue):
		pass

	def format(self, value):
		return "%g" % value

class LogLabelFormatter(LabelFormatter):
	"""Formats the value using (10**value)
	This gives log scales
	"""
	def init(self, minvalue, maxvalue):
		pass

	def format(self, value, mi, ma):
		if mi < -4 or ma > 4:
			#return "%1.0e" % (10**value)
			return "10\u%i\d" % value
		else:
			return "%g" % (10**value)

class UserLabelFormatter(LabelFormatter):
	"""Formats the value using a user-defined
	conversion string. (e.g. '%1.2e'), defined in Axis.labeling.
	"""
        def init(self, minvalue, maxvalue):
		pass

	def format(self, value, conversionString):
		return conversionString % value


class Axis(PlotObject):
	"""A line with tickmarks and subticksmarks and labels at the end of the tickmarks

	Ticks:
	A line is drawn from the begin to the end point. Depending on the Axis.intersect
	member the x or y elements of the world coordinates. The intersection of the
	tick and the axis line is marked with a tickmark and labeled using the
	labelformatter.
	e.g If world coordinates go from (0, 0)-(10, 200) and an axis is drawn, with
	ticks = [1,5,10] and intersect = "x". At any point on the axis' line where
	the x element equals 1,5 or 10, a tickmark and label is drawn.
	The length of the tickmark depends in Axis.ticklength, and is expressed in
	Axis.units or the default units of the Device.
	Note that the ticklength may be nagative, to make the ticks go inwards.

	Subticks:
	The same for subticks, but they are not labeled, and the default length is
	smaller.

	Labeling:
	At every tick(not subticks) a label is placed. The string placed is determined
	by the labelformatter, it should be a subclass of LabelFormatter.
	If halign or valign aren't set, alignments are chosen based on the angle between
	the tickline and the textangle.

	Properties:
	- p1
	- p2
	- ticks
	- subticks
	- labels
	- ticklength
	- subticklength 
	- labeloffset
	- color
	- labelcolor
	- linewidth
	- intersect
	- units
	- textangle
	- fontsize
	- labelformatter
	- labelstyle
	- tickstyle
	- logAxis
	- valign
	- halign        
	- matrix
	"""
	ticks = autoproperty(default=[])
	subticks = autoproperty(default=[])

	labels = autoproperty(default=[])

	ticklength = numberproperty(default=-4)
	subticklength = numberproperty(default=-2)

	labeloffset = numberproperty()
	p1 = pointproperty()
	p2 = pointproperty()
	color = colorproperty()
	labelcolor = colorproperty()
	linewidth = linewidthproperty()
	intersect = stringproperty(docstring="interect with 'x' or 'y' element")
	units = stringproperty()
	textangle = numberproperty(default=0)
	fontsize = fontsizeproperty()
	
	labelformatter = autoproperty(default=DefaultLabelFormatter())				
	labelstyle = stringproperty(default="normal")
	tickstyle = stringproperty(default="normal")
	logAxis =  booleanproperty(default=False)
	
	valign = stringproperty()
	halign = stringproperty()

	matrix = matrixproperty()


	def __init__(self, p1=None, p2=None, side=None, defaults=None, **properties):
		"""Constructor

		Use p1 and p2 or side to define where the Axis should be drawn.
		axis = Axis(side="bottom")
		or
		axis = Axis(p1=(0,0), p2=(1,0))

		When using p1 and p2, the 'outside'(where the tickmarks will be placed)
		is determined by the direction the points go. Seen from p1 to p2, the
		right side is outside.
		axis = Axis(p1=(0,0), p2=(0,1))	# left axis with tickmarks and labels
										# at the right, probably not what you want
		axis = Axis(p1=(0,1), p2=(0,0))	# left axis with tickmarks and labels
										# at the left

		see Axis.example() for an example
		and Axis.test() for an examle using a labelformatter


		Arguments:
		p1 -- location in normalised coordinates
		p2 -- idem
		side -- "bottom", "top", "left" or "right"
		"""
		PlotObject.__init__(self, defaults=defaults, properties=properties)
		if side == None:
			if p1 == None or p2 == None:
				raise Exception, "invalid arguments, use p1 and p2, or side"
			self.p1 = p1
			self.p2 = p2
		else:
			if side == "left":
				self.p1 = (0, 1)
				self.p2 = (0, 0)
			elif side == "bottom":
				self.p1 = (0, 0)
				self.p2 = (1, 0)
			elif side == "right":
				self.p1 = (1, 0)
				self.p2 = (1, 1)
			elif side == "top":
				self.p1 = (1, 1)
				self.p2 = (0, 1)
			else:
				raise Exception, "invalid side: " +str(side)

		angle = todegrees((Vector(self.p2) - Vector(self.p1)).angle())
		octant = int(angle / 45) + 1
		if self.intersect == None:
			if octant in [1,8,4,5]:
				self.intersect = "x"
			else:
				self.intersect = "y"

		if self.labeloffset == None:
			self.labeloffset = abs(self.ticklength) + 2
		#if not properties.has_key("labelcolor"):
		#	self.labelcolor = self.color

	def plot(self, dev):
		clipping = dev.getClipping()
		dev.setClipping(False)
		dev.pushWorldMatrix(self.matrix)

		if self.labelcolor == None:
			self.labelcolor = self.color

		worldmatrix = dev.getWorldMatrix()
		invworldmatrix = worldmatrix.inverse()
		dev.pushWorldMatrix(invworldmatrix)

		start = invworldmatrix * self.p1
		end = invworldmatrix * self.p2
		line = _AxisLine(start, end)

		tickdir = Matrix.rotate(toradians(-90)) * (Vector(self.p2)-Vector(self.p1)).unit()

		gfxline = Line(self.p1, self.p2, color=self.color, linestyle="normal", linewidth=self.linewidth)
		gfxline.plot(dev)
		tickangle = tickdir.angle()
		scale = dev.getUnitMatrix(self.units)

		if self.ticks:
			self.labelformatter.init(min(self.ticks), max(self.ticks))
			self.logAxis=(self.tickstyle=="log")
		if self.intersect == "y":
			for tick in self.subticks:
				pos = line.ytick(tick,self.logAxis)
				if pos:
					pos = worldmatrix * pos
					tickend = pos + scale * tickdir.scale(self.subticklength)
					gfxline = Line(pos, tickend, color=self.color, linestyle="normal", linewidth=self.linewidth)
					gfxline.plot(dev)
			for tick in self.ticks:
				pos = line.ytick(tick)
				if pos:
					pos = worldmatrix * pos
					tickend = pos + scale * tickdir.scale(self.ticklength)
					gfxline = Line(pos, tickend, color=self.color, linestyle="normal", linewidth=self.linewidth)
					gfxline.plot(dev)
					if self.labels and self.labelstyle.lower()!="none":
						self.label(tick, pos + scale * tickdir.scale(self.labeloffset), dev, tickangle)
		elif self.intersect == "x":
			for tick in self.subticks:
				pos = line.xtick(tick,self.logAxis)
				if pos:
					pos = worldmatrix * pos
					tickend = pos + scale * tickdir.scale(self.subticklength)
					gfxline = Line(pos, tickend, color=self.color, linestyle="normal", linewidth=self.linewidth)
					gfxline.plot(dev)
			for tick in self.ticks:
				pos = line.xtick(tick)
				if pos:
					pos = worldmatrix * pos
					tickend = pos + scale * tickdir.scale(self.ticklength)
					gfxline = Line(pos, tickend, color=self.color, linestyle="normal", linewidth=self.linewidth).plot(dev)
					if self.labels and self.labelstyle.lower()!="none":
						self.label(tick, pos + scale * tickdir.scale(self.labeloffset), dev, tickangle)
		else:
			raise Exception, "unknown intersect: " +str(intersect)

		dev.popWorldMatrix()
		dev.popWorldMatrix()
		dev.setClipping(clipping)
		return

	def label(self, tick, position, dev, tickangle):
		if self.labelstyle.lower() not in ['log','none','normal']:
			if self.labelstyle.lower()[0] != '%':
				self.labelstyle = '%'+self.labelstyle.lower()
			try:
				self.labelstyle.lower() % 1
			except ValueError:
				textstr = self.labelformatter.format(tick)
			else:
				self.labelformatter = UserLabelFormatter()
				textstr = self.labelformatter.format(tick,self.labelstyle.lower())
		elif self.labelstyle.lower() == 'log':  
			self.labelformatter = LogLabelFormatter()
			textstr = self.labelformatter.format(tick,min(self.ticks), max(self.ticks))
		else:
			textstr = self.labelformatter.format(tick)

		textangle = self.textangle
		halign = self.halign
		if halign == None:
			halign = self.getHalign(tickangle, textangle)

		valign = self.valign
		if valign == None:
			valign = self.getValign(tickangle, textangle)

		text = Text(textstr, position, textangle=textangle, color=self.labelcolor, halign=halign, valign=valign, fontsize=self.fontsize)
		text.plot(dev)

	def getHalign(self, tickangle, textangle):
		"""
		Return the horizontal alignment based on the (-22.5 degrees rotated)'octant'
		the tickangle is in respect to the baseline of the text.

		The first octant starts at -22.5 and ends at 22.5 degrees.

		octant:
			1,2,8:		"left"
			3,7:		"center"
			4,5,6:		"right"

		"""
		tickangle = (todegrees(tickangle) + 360) % 360
		textangle = (todegrees(textangle) + 360) % 360
		angle = (tickangle-textangle + 360.0) % 360
		octant = int((angle+22.5) / 45) + 1
		if octant in [1,2,8]:
			return "left"
		elif octant in [3,7]:
			return "center"
		elif octant in [4,5,6]:
			return "right"
		return "center"


	def getValign(self, tickangle, textangle):
		"""
		Return the vertical alignment based on the (-22.5 degrees rotated)'octant'
		the tickangle is in respect to the baseline of the text.

		The first octant starts at -22.5 and ends at 22.5 degrees.

		octant:
			2,3,4:		"bottom"
			1,5:		"center"
			6,7,8:		"top"

		"""
		tickangle = (todegrees(tickangle) + 360) % 360
		textangle = (todegrees(textangle) + 360) % 360
		angle = (tickangle-textangle + 360.0) % 360
		octant = int((angle+22.5) / 45) + 1
		if octant in [2,3,4]:
			return "bottom"
		elif octant in [1,5]:
			return "center"
		elif octant in [6,7,8]:
			return "top"
		return "center"

	def example():
		import Kplot

		print "Axis example"
		print
		print 'There are two ways to create an Axis.'
		print 'First of all there are four pre-programmed sides:'
		print 'bottom, top, left and right'
		print 'You can also plot an axis between two points'
		print
		print 'two examples:'
		print 'bottomaxis = Kplot.Axis(side="bottom", ticks=range(0, 16, 3), labels=range(0, 16, 3))'
		bottomaxis = Kplot.Axis(side="bottom", ticks=range(0, 16, 3), labels=range(0, 16, 3))
		leftaxis = Kplot.Axis(p1=(.1,1),p2=(0,0), ticks=range(11), labels=range(11))
		dev = Kplot.createDevice()
		panel = dev.createPanel()
		panel.add(bottomaxis)
		panel.setWorld(p1=(0, 0), p2=(15, 10))
		dev.output()
		raw_input(' Type <RETURN> for next page:')
		print 'leftaxis = Kplot.Axis(p1=(.1,1),p2=(0,0), ticks=range(11), labels=range(11))'
		panel.add(leftaxis)		
		dev.output()
		dev.close()

	example = staticmethod(example)

	def test():
		import Kplot
		import Numeric

		x = range(0, -21, -4)
		y = range(12)

		bottomaxis = Kplot.Axis(p1=(0, 0), p2=(1,0.5), ticks=x, subticks=range(0, -20, -1), textangle=toradians(-90), labels=x)
		leftaxis = Kplot.Axis(p1=(0.8, 1), p2=(0,0), ticks=y, subticks=Numeric.arange(0,10.1, 0.5), textangle=toradians(180), labelformatter=MonthLabelFormatter(),labels=y)
		grid = Kplot.Grid(yticks=leftaxis.ticks, xticks=bottomaxis.ticks)
		axes = []
		axes.append(Kplot.Axis(side="bottom", ticks=x, labels=x))
		axes.append(Kplot.Axis(side="top", ticks=x, labels=x))
		axes.append(Kplot.Axis(side="left", ticks=y, labels=y))
		axes.append(Kplot.Axis(side="right", ticks=y, labels=y))

		dev = Kplot.createDevice()
		panel = dev.createPanel()
		panel.add(grid, bottomaxis, leftaxis, axes)
		panel.setWorld(p1=(0, 0), p2=(-20, 11))
		dev.output()
		dev.close()

	test = staticmethod(test)



