"""Objects that correspond directly to plot methods of Device"""
from plotobject import PlotObject
from properties import *
from autoproperty import *
from Kplot import ColorMap, toradians
import types
import Numeric
import math
"""
Contains basic plotobjects that correspond directly to Kplot.Device plot methods
"""


class Polyline(PlotObject):
	"""A Polyline draws one of more connected lines
	The input are two array with points

	properties:
	- color
	- linestyle
	- linewidth
	- matrix
	- units
	"""
	color = colorproperty()
	linestyle = linestyleproperty()
	linewidth = linewidthproperty()
	x = arrayproperty()
	y = arrayproperty()
	matrix = matrixproperty()
	units = stringproperty()

	def __init__(self, x, y, defaults=None, **properties):
		"""Constructor

		Arguments:
		x -- a sequence with all x elements
		y -- Idem, but for y

		"""
		PlotObject.__init__(self, defaults=defaults, properties=properties)
		self.x = x
		self.y = y

	def plot(self, dev):
		dev.pushWorldMatrix(self.matrix)
		dev.plotPolyline(self.x, self.y, color=self.color, linestyle=self.linestyle, linewidth=self.linewidth, units=self.units)
		dev.popWorldMatrix()

	def example():
		import Kplot

		print "Polyline example"
		print
		print 'A Polyline needs two arrays of datapoints'
		print 'for instance'
		print 'x = range(6)'
		print 'y = [20,0,40,30,10,20]'
		print
		print 'The Polyline is then made by'
		print 'polyline = Kplot.Polyline(x, y)'
		x = range(6)       
		y = [20,0,40,30,10,20]
		polyline = Kplot.Polyline(x, y)
		print
		print 'which results in:'
		dev = Kplot.createDevice()
		panel = dev.createPanel()
		panel.add(polyline)
		panel.setWorld(p1=(0, 0), p2=(5, 40))
		dev.output()
		raw_input(' Type <RETURN> for next page:')
		print
		print 'Polylines have a lot of properties, so the plotobject'
		print 'can be extended in, for instance:'
		print 'polyline = Kplot.Polyline(x, y, linewidth=2, linestyle="dashed", color="red")'
		print
		polyline.linewidth=2
		polyline.linestyle="dashed"
		polyline.color="red"
		dev.output()
		dev.close()

	example = staticmethod(example)


	def test():
		import Kplot
		
		dev = Kplot.createDevice()
		panel = dev.createPanel()
		left, right = panel.split(0.6, "vertical", spacing=0.01)
		left = left.mosaic(1, 5, spacing=0.02)
		right = right.mosaic(1, 5, spacing=0.02)

		defaults = {"halign":"left", "valign":"center", "fontsize":12}

		left[0].add(Kplot.Text("2 mm / dotted",(0, 0.33), defaults=defaults))
		right[0].add(Kplot.Polyline([0, 1],  [0.33,0.33], linestyle="dotted", units="mm", linewidth=2))
		left[0].add(Kplot.Text("5 mm / normal",(0, 0.66), defaults=defaults))
		right[0].add(Kplot.Polyline([0, 1],  [0.66, 0.66], linestyle="normal", units="mm", linewidth=5))

		left[1].add(Kplot.Text("0.5 cm / normal",(0, 0.33), defaults=defaults))
		right[1].add(Kplot.Polyline([0, 1],  [0.33,0.33], linestyle="normal", units="cm", linewidth=0.5))
		left[1].add(Kplot.Text("0.1 cm / dashed",(0, 0.66), defaults=defaults))
		right[1].add(Kplot.Polyline([0, 1],  [0.66,0.66], linestyle="dashed", units="cm", linewidth=0.1))


		left[2].add(Kplot.Text("1/72 inch / normal",(0, 0.33), defaults=defaults))
		right[2].add(Kplot.Polyline([0, 1],  [0.33, 0.33], linestyle="normal", units="inch", linewidth=1.0/72))
		left[2].add(Kplot.Text("0.2 inch / dashed",(0, 0.66), defaults=defaults))
		right[2].add(Kplot.Polyline([0, 1],  [0.66, 0.66], linestyle="dashed", units="inch", linewidth=0.2))


		left[3].add(Kplot.Text("1 point / dotted",(0, 0.33), defaults=defaults))
		right[3].add(Kplot.Polyline([0, 1],  [0.33,0.33], linestyle="dotted", units="points", linewidth=1))
		left[3].add(Kplot.Text("4 points / dashed",(0, 0.66), defaults=defaults))
		right[3].add(Kplot.Polyline([0, 1],  [0.66,0.66], linestyle="dashed", units="points", linewidth=4))

		left[4].add(Kplot.Text("1/20 normalised / normal",(0, 0.33), defaults=defaults))
		right[4].add(Kplot.Polyline([0, 1],  [0.33,0.33], linestyle="normal", units="normalised", linewidth=1.0/20))
		left[4].add(Kplot.Text("1/40 normalised / dashed",(0, 0.66), defaults=defaults))
		right[4].add(Kplot.Polyline([0, 1],  [0.66, 0.66], linestyle="dashed", units="normalised", linewidth=1.0/40))

		dev.output()
		dev.close()

	test = staticmethod(test)

class Polygon(PlotObject):
	"""A Polygon fills the interior of the it's enclosing region

	The regions is defined by the x and y elements, and the length of both should
	be >= 3. The region is automaticly closed, so the begin and end points don't have
	to match up.

	When filling complex(non convex) polygons, the odd rule is used to find out if
	the area should be filled. Meaning, that if a line from infinity intersects an
	odd number of edges, it is inside the polygon.

	A Polyline is drawn through the points, but is not seen if linestyle is set to
	"noline"

	properties:
	- color
	- linestyle
	- linewidth
	- fillstyle
	- matrix 
	- units 
	"""
	color = colorproperty()
	linestyle = linestyleproperty()
	linewidth = linewidthproperty()
	fillstyle = fillstyleproperty()
	x = arrayproperty()
	y = arrayproperty()
	matrix = matrixproperty()
	units = stringproperty()

	def __init__(self, x, y, defaults=None, **properties):
		"""Constructor

		Arguments:
		x -- a sequence with all x elements
		y -- Idem, but for y

		"""
		#properties.update({"x":x, "y":y})
		PlotObject.__init__(self, defaults=defaults, properties=properties)
		self.x = x
		self.y = y

	def plot(self, dev):
		dev.pushWorldMatrix(self.matrix)
		dev.plotPolygon(self.x, self.y, color=self.color, linewidth=self.linewidth, linestyle=self.linestyle, fillstyle=self.fillstyle, units=self.units)
		dev.popWorldMatrix()
	
	def example():
		import Kplot
		print 'Polygon example'
		print
		print 'A Polygon needs two arrays of datapoints'
		print 'for instance'
		print 'x = [1, 4, 4, 1]'
		print 'y = [1, 1, 4, 4]'
		print
		print 'The Polygon is then made by'
		print 'polygon = Kplot.Polygon(x, y)'
		x = [1, 4, 4, 1]
		y = [1, 1, 4, 4]
		polygon = Kplot.Polygon(x, y)
		print
		print 'which results in:'
		dev = Kplot.createDevice()
		panel = dev.createPanel()
		panel.add(polygon)
		panel.setWorld(p1=(0, 0), p2=(5, 5))
		dev.output()
		raw_input(' Type <RETURN> for next page:')
		print
		print 'Polygons have a lot of properties, so the plotobject'
		print 'can be extended in, for instance:'
		print 'polygon = Kplot.Polygon(x, y, fillstyle="crosshatched", color="blue")'
		print
		polygon.fillstyle="crosshatched"
		polygon.color="blue"
		dev.output()
		dev.close()

	example = staticmethod(example)


	def test():
		import Kplot

		dev = Kplot.createDevice()
		mpanel = dev.createPanel(p1=(0.1, 0.1), p2=(0.9,0.9))

		linestyles = ['noline', 'normal', 'dashed']
		fillstyles = ['nofill', 'solid', 'hatched', 'crosshatched']
		panels = mpanel.mosaic(4,5)

		for i in range(3):
			panels[i+1][0].setWorld((-10, -10), (0, 0))
			panels[i+1][0].add(Kplot.Text(linestyles[i], (-5, -5), fontsize=12))
		for i in range(4):
			panels[0][i+1].add(Kplot.Text(fillstyles[i], (0, 0.5), fontsize=12, halign="left", valign="top"))

		linestylenr = 0
		fillstylenr = 0
		for column in panels[1:]:
			linestyle = linestyles[linestylenr]
			for panel in column[1:]:
				fillstyle = fillstyles[fillstylenr]
				rect = Kplot.Polygon([0.1, 0.9, 0.9, 0.1], [0.1, 0.1, 0.9, 0.9], fillstyle=fillstyle, linestyle=linestyle)
				panel.add(rect)
				fillstylenr += 1
			fillstylenr = 0
			linestylenr +=1

		dev.output()
		dev.close()

	test = staticmethod(test)

class Text(PlotObject):
	"""Draws a string

	The valign and halign(v stands for vertical, h for horizontal) define the
	meaning of location(the point where the text is drawn).
	A point in the bounding box of the text is aligned with the position.
	e.g. when halign is "right" and valign is "top", the top right box will
	correspond to the location, even when the text is rotated(using textangle)

	properties:
	- text
	- location      
	- color
	- bgcolor
	- font
	- fontsize 
	- linewidth
	- textangle
	- valign
	- halign
	- units
	- weight
	"""
	text = stringproperty()
	location = pointproperty()

	color = colorproperty()
	bgcolor = bgcolorproperty()
	font = fontproperty()
	fontsize = fontsizeproperty()
	linewidth = linewidthproperty()
	textangle = numberproperty(default=0,docstring="angle in radians, going anticlockwise, 0 being horizontal")
	#just = autoproperty(typelist = [types.StringType, types.FloatType, types.IntType], default = "left", docstring="see Device.plotText")
	valign = stringproperty(default="center")
	halign = stringproperty(default="center")
	matrix = matrixproperty(docstring="only the location of the text will be transformed, not the orientation")
	units = stringproperty(default="points")
	weight = fontweightproperty()

	def __init__(self, text, location, defaults=None, **properties):
		"""Constructor

		Arguments:
		text -- the text string
		location -- in the form on a (x,y) tuple

		"""
		PlotObject.__init__(self, defaults=defaults, properties=properties)
		self.text = text
		self.location = location

	def plot(self, dev):
		dev.pushWorldMatrix(self.matrix)
		dev.plotText(self.text, self.location, font=self.font, fontsize=self.fontsize, weight=self.weight,
			color=self.color, bgcolor=self.bgcolor, textangle=self.textangle,
			 valign=self.valign, halign=self.halign, units=self.units)
		dev.popWorldMatrix()

	def example():
		import Kplot
		print "Text example"
		print
		print 'Text needs a text string and a position tuple'
		print 'for instance'
		print 'pos = (0.5, 0.5)'
		print 'tex = "Test"'
		print
		print 'The Text object is then made by'
		print 'text = Kplot.Text(tex,pos)'
		print
		print 'which results in:'
		pos = (0.5, 0.5)
		tex = "Test"
		text = Kplot.Text(tex,pos)
		dev = Kplot.createDevice()
		panel = dev.createPanel()
		panel.add(text)
		dev.output()
		raw_input(' Type <RETURN> for next page:')
		print
		print 'Text has a lot of properties, so the plotobject'
		print 'can be extended in, for instance:'
		print 'text = Kplot.Text(tex,pos, color="blue", bgcolor="red", valign="top", fontsize=2, units="cm", weight=0.1)'
		print
		text.color="blue"
		text.bgcolor="red"
		text.valign="top"
		text.fontsize=2
		text.units="cm"
		text.weight=0.1
		dev.output()
		dev.close()

	example = staticmethod(example)


	def test():
		import Kplot

		dev = Kplot.createDevice()
		panel = dev.createPanel()
		left, right = panel.split(0.6, "vertical", spacing=0.01)
		left = left.mosaic(1, 4, spacing=0.02)
		right = right.mosaic(1, 4, spacing=0.02)

		defaults = {"halign":"left", "valign":"center", "fontsize":12, "weight":1}

		left[0].add(Kplot.Text("italic 12pt, weight=1.5",(0, 0.66), defaults=defaults))
		left[0].add(Kplot.Text("halign='left', valign='top'",(0, 0.33), defaults=defaults))
		right[0].add(Kplot.Text("test", (0.5, 0.5), font="italic", fontsize=12, weight=1.5, halign="left", valign="top", bgcolor="white"))
		right[0].add(Kplot.Marker((0.5, 0.5), Kplot.markertype.x, color="red"))

		left[1].add(Kplot.Text("normal 10pt, weight=1",(0, 0.66), defaults=defaults))
		left[1].add(Kplot.Text("halign='center', valign='bottom'",(0, 0.33), defaults=defaults))
		right[1].add(Kplot.Text("test", (0.5, 0.5), font="normal", fontsize=10, weight=1, halign="center", valign="bottom", bgcolor="white"))
		right[1].add(Kplot.Marker((0.5, 0.5), Kplot.markertype.x, color="red"))

		left[2].add(Kplot.Text("script 45pt, weight=2",(0, 0.66), defaults=defaults))
		left[2].add(Kplot.Text("halign='center', valign='center'",(0, 0.33), defaults=defaults))
		right[2].add(Kplot.Text("test", (0.5, 0.5), font="script", fontsize=45, weight=2, halign="center", valign="center", bgcolor="white"))
		right[2].add(Kplot.Marker((0.5, 0.5), Kplot.markertype.x, color="red"))

		left[3].add(Kplot.Text("roman 50pt, weight=4",(0, 0.66), defaults=defaults))
		left[3].add(Kplot.Text("halign='right', valign='top'",(0, 0.33), defaults=defaults))
		right[3].add(Kplot.Text("test", (0.5, 0.5), font="roman", fontsize=50, weight=4, halign="right", valign="top", bgcolor="white"))
		right[3].add(Kplot.Marker((0.5, 0.5), Kplot.markertype.x, color="red"))

		dev.output()
		dev.close()

	test = staticmethod(test)

class Image(PlotObject):
	"""Image draws an intensity image with colors defined by the colormap

	As input it requires a 2D array.
	This array is then mapped using the given colomap (default rainbow).
	By default the axes of a box surrounding an Image start at 0,0 and
	go to the number of pixels on each side.
	The 0,0 point can be changed by setting Image.origin.
	By default the colormap is interpolated lineary, however an other
	interpolation function can be given to the colormap. See Kplot.Colormap
	for more information.

	properties:
	- data
	- origin 
	- colormap 
	- datamin
	- datamax
	- matrix
	"""
	matrix = matrixproperty()
	data = arrayproperty()
	origin = arrayproperty(default=(0,0))
	colormap = functionautoproperty()
	datamin = numberproperty(docstring="The minimum value that is used for the colormapping, values below the minimum are drawn in the same Color as the minimum value. Default is the minimum value found in the data.")
	datamax = numberproperty(docstring="Same as datamin, but now for the maximum value. Default is the maximum value found in the data.")
	function = functionautoproperty(default=lambda x: x)

	def __init__(self, data, defaults=None, **properties):
		"""Constructor

		Arguments:
		data -- A 2-dimensional sequence array containing values which will be
			mapped to a Color using the colormap and function
		"""
		PlotObject.__init__(self, defaults=defaults, properties=properties)
		self.data = data
		if self.colormap == None:
			self.colormap = ColorMap.rainbow
		if self.datamin == None or self.datamax == None:
			flat = Numeric.ravel(self.data)
			if self.datamin == None:
				self.datamin = min(flat)
			if self.datamax == None:
				self.datamax = max(flat)
		
		if self.origin == None:
			self.origin = (0,0)
		delx,dely=self.origin
		delx+=.5
		dely+=.5
		self.origin = delx,dely
		
	def plot(self, dev):
		dev.pushWorldMatrix(self.matrix)
		dev.plotImage(data=self.data, colormap=self.colormap, datamin=self.datamin, datamax=self.datamax, function=self.function, origin=self.origin)
		dev.popWorldMatrix()

	def getWidth(self):
		"""Return the width of the Image"""
		return len(self.data[0])

	def getHeight(self):
		"""Return the height of the Image"""
		return len(self.data)

	def example():
		import Kplot
		import Numeric

		print "Image example"
		print
		print 'all that Image needs as input is a 2D array (i.e. a matrix)'
		print 'In this example we use:'
		print 'data = []'
		print 'for i in range(4):'
		print '      data.append((Numeric.arange(0,20)+i)%20)'
		data = []
		for i in range(4):
			data.append((Numeric.arange(0,20)+i)%20)
		print
		print 'The image is then created by:'
		print 'image = Kplot.Image(data)'
		print
		print 'to fit the image in the panel use:'
		print 'panel.setWorldImage(image)'
		print
		image = Kplot.Image(data)
		dev = Kplot.createDevice()
		panel = dev.createPanel()
		panel.add(image)
		p1 = (0.5, 0.5)
		p2 = (20.5, 4.5)
		panel.setWorldImage(image)
		dev.output()
		dev.close()
		print

	example = staticmethod(example)


	def test():
		import Kplot
		import Numeric

		print "There should be a blue bar at the left, and a red bar at the right of about 5% of the total image"
		def testcolormap(value):
			if value < 0.01:
				return Kplot.Color.red
			elif value > 0.99:
				return Kplot.Color.blue
			else:
				return Kplot.Color.green

		data = []
		for i in range(4):
			data.append((Numeric.arange(0,100)))

		image = Kplot.Image(data, datamin=5, colormap=testcolormap, datamax=95, function=lambda x: 1-x)

		dev = Kplot.createDevice()
		panel = dev.createPanel()
		panel.add(image)
		p1 = (0.5, 0.5)
		p2 = (100.5, 4.5)
		panel.setWorld(p1, p2)
		dev.output()
		dev.close()

	test = staticmethod(test)

class Contours(PlotObject):
	"""Contours draws a contourmap of a 2D array or Kplot.Image

	Arguments:
	data   -- A 2-dimensional sequence array containing values which will be
	          mapped with contours, or a Kplot.Image
	levels -- Contourlevel(s) to be mapped. Can be either a list or a single
	          value
	other properties:
	color          -- string or list of strings containing the color(s) of the
	                  contour(s). If less colors the levels are given, the last
			  color is appended.
	linestyle      -- string or list of strings containing the linestyle(s) of
	                  the contour(s). Last style is appended if list is too
			  short. 
	linewidth      -- string or list of strings containing the linewidth(s) of
	                  the contour(s).
	origin         -- origin of contourmap in panel. Similar to
	                  Kplot.Image.origin. If data is an Kplot.Image,
			  Kplot.Image.origin is used for Kplot.Contours.origin
	units          -- units used for linewidth, fontsize, etc.
	labeling       -- boolean, switches contour labeling on/off
	labelstart     -- cel number to start labeling (just play with it to get
	                  good result)
	labelinterval  -- interval of label in cels (just play with it to get good
	                  result)
	labelcolor     -- string or list of strings containing the color(s) of the
	                  contour label(s). If less colors the levels are given,
			  last color is appended.
	font           -- font used for contour label(s)
	fontsize       -- fontsize used for contour label(s)
	labellinewidth -- linewidth used for contour label(s)
	blank          -- integer or float specifying pixel value to be blanked
	
	For more explanation run Contours.example()
	"""
	matrix         = matrixproperty()
	origin         = arrayproperty(default=(0,0))
	levels         = arrayproperty(default=[])
	labeling       = booleanproperty(default=False)
	color          = autoproperty(typelist = [types.StringType,types.ListType], default='green')
	linestyle      = autoproperty(typelist = [types.StringType,types.ListType], default='normal')
	linewidth      = autoproperty(typelist = [types.FloatType,types.ListType,types.IntType], default=0.5)
	units          = autoproperty(typelist = [types.NoneType,types.ListType,types.StringType], default=None)
	labelstart     = autoproperty(typelist = [types.ListType,types.IntType], default=10)
	labelinterval  = autoproperty(typelist = [types.ListType,types.IntType], default=20)
	labelcolor     = autoproperty(typelist = [types.StringType,types.ListType])
	font           = autoproperty(typelist = [types.StringType,types.ListType], default='normal')
	fontsize       = autoproperty(typelist = [types.FloatType,types.ListType,types.IntType], default=4)
	labellinewidth = autoproperty(typelist = [types.FloatType,types.ListType,types.IntType])
	blank          = numberproperty()
	
	def __init__(self, data, defaults=None, **properties):
		from Kplot import Image
		"""Constructor
		"""
		PlotObject.__init__(self, defaults=defaults, properties=properties)

		self.data = data
		if isinstance(self.data, Image):
			self.origin = self.data.origin
			self.data = self.data.data
		elif type(self.data) != type(Numeric.array([])):
			raise TypeError('data must be an array or Kplot.Image')

		if self.levels==[]:
			raise IndexError('Contours.levels should contain at least one level')

		if not self.labelcolor:
			self.labelcolor=self.color
		if not self.labellinewidth:
			self.labellinewidth=self.linewidth
		if not self.blank:
			self.blank = min(Numeric.array(self.data).flat)

		props = [('self.color',self.color),('self.linestyle',self.linestyle),('self.linewidth',self.linewidth),
			 ('self.units',self.units),('self.labelstart',self.labelstart),('self.labelinterval',self.labelinterval),
			 ('self.labelcolor',self.labelcolor),('self.font',self.font),('self.fontsize',self.fontsize),
			 ('self.labellinewidth',self.labellinewidth)]

		for name,prop in props:
			if type(prop) != types.ListType:
				prop = [prop]
				cmd = name+'='+str(prop)
				exec(compile(cmd,'errorLog','exec'))

 		props = [('self.color',self.color),('self.linestyle',self.linestyle),('self.linewidth',self.linewidth),
 			 ('self.units',self.units),('self.labelstart',self.labelstart),('self.labelinterval',self.labelinterval),
 			 ('self.labelcolor',self.labelcolor),('self.font',self.font),('self.fontsize',self.fontsize),
 			 ('self.labellinewidth',self.labellinewidth)]

		for name,prop in props:
			cmd1 = 'while len('+name+') < len(self.levels): '+name+'.append(prop[-1])'
			exec(compile(cmd1,'errorLog','exec'))
		
		delx,dely=self.origin
		delx+=.5
		dely+=.5
		self.origin = delx,dely				

	def plot(self, dev): 
		if str(dev)[7:13] != 'pgplot':
			raise NotImplementedError('Countours only work on pgplot device')
		else:
			dev.pushWorldMatrix(self.matrix)
			for i in range(len(self.levels)):
				dev.plotContour(self.data,level=self.levels[i],color=self.color[i],linestyle=self.linestyle[i],
						linewidth=self.linewidth[i],origin=self.origin, units=self.units[i],
						labeling=self.labeling, labelstart=self.labelstart[i],blank = self.blank,
						labelinterval=self.labelinterval[i],labelcolor=self.labelcolor[i],
						font=self.font[i],fontsize=self.fontsize[i],labellinewidth=self.labellinewidth[i])
			dev.popWorldMatrix()

	def example():
		import Kplot
		import Numeric
		print
		print "  Contour example"
		print
		print 'Making a contour map starts with creating an Image.'
		print '(In this case 2 nice gaussians)'
		print
		def fie(i,j,size):
			val = Numeric.exp(-((3/size)*(i-.3*size))**2-((3/size)*(j-.3*size))**2)+.75*Numeric.exp(-((5/size)*(i-.7*size))**2-((5/size)*(j-.7*size))**2)
			return val
		size          = 100.
		data          = Numeric.array([[fie(i,j,size) for i in range(int(size))] for j in range(int(size))])
		data          = Kplot.Image(data,colormap=Kplot.ColorMap.whiteblack)
		box = Kplot.Box(labeling='none',ticks='none')
		dev = Kplot.createDevice()
		panel = dev.createPanel()
		panel.setWorldImage(data)
		panel.add(data,box)
		dev.output()
		print 'After making the Image a contourmap is made with the'
		print 'Kplot.Contours command'
		print
		print 'this command takes at least 2 arguments:'
		print '  data   - data in the form of a 2D array or a Kplot.Image'
		print '  levels - the contour level(s) in the form of an array'
		print '                                      or a single value'
		print
		print 'This example uses a Kplot.Image and for the levels'
		print 'level = [.3,.56,.8,.95]'
		print
		print 'Then the contour map is created by:'
		print 'Contourmap = Kplot.Contours(data,levels=level)'
		print 
		raw_input(' Type <RETURN> for next page:')
		level         = [.3,.56,.8,.95]
		C = Kplot.Contours(data,levels=level)
		panel.add(C)
		dev.output()
		print
		print 'As can be seen, all contours are green'
		print 'ofcourse this can be changed:'
		print
		print """colors = ['red','green','yellow','cyan']"""
		print 'Contourmap.color=colors'
		print
		raw_input(' Type <RETURN> for next page:')
		color         = ['red','green','yellow','cyan']
		C.color=color
		print
		print 'Contours can also be labeled by setting'
		print 'Countour.labeling to True'
		print
		raw_input(' Type <RETURN> for next page:')
		C.labeling=True
		print
		print 'Other properties to play with are:'
		print '- linestyle'
		print '- linewidth'
		print '- origin'
		print '- units'
		print '- labelstart'
		print '- labelinterval'
		print '- labelcolor'
		print '- font'
		print '- fontsize'
		print '- labellinewidth'
		print '- blank'
		print
		print 'Good Luck!'
		dev.close()
	example = staticmethod(example)


	def test():
		import Kplot
		import Numeric
		print
		print "There should be a four concentric circles"
		print "colors should be red, green, yellow and cyan (inwards)"
		print
		def fie(i,j,size):
			val = Numeric.exp(-((5/size)*(i-.5*size))**2-((5/size)*(j-.5*size))**2)
			return val
		size          = 100.
		data          = Numeric.array([[fie(i,j,size) for i in range(int(size))] for j in range(int(size))])
		data          = Kplot.Image(data,colormap=Kplot.ColorMap.whiteblack)
		box = Kplot.Box(labeling='none',ticks='none')
		dev = Kplot.createDevice()
		panel = dev.createPanel()
		panel.setWorldImage(data)
		level         = [.05,.3,.6,.8]
		C = Kplot.Contours(data,levels=level)
		color         = ['red','green','yellow','cyan']
		C.color=color
		panel.add(C,box)
		dev.output()
		dev.close()
	test = staticmethod(test)

