"""Box is a compound class, consists of 4 axes and a grid, does autolabeling"""
from plotobject import PlotObject
from properties import *
from axis import Axis
from grid import Grid
from math import *
from Numeric import *

def nextpow10(n):
	"""Return the next power of 10"""
	if n == 0:
		return 0
	else:
		return ceil(log(abs(n))/log(10))

def fround(value, mod):
	if mod == 0:
		return value
	else:
		return int(value/mod) *mod

def magicnr(value, error):
	"""Return a number that looks 'nice', with a maximum error"""
	magics = [	(10 ** (nextpow10(error))),
				(10 ** (nextpow10(error))) / 2.0,
				(10 ** (nextpow10(error))) / 4.0,
				(10 ** (nextpow10(error))) / 10.0,
				(10 ** (nextpow10(error))) / 20.0,
				(10 ** (nextpow10(error))) / 40.0,
				(10 ** (nextpow10(error))) / 100.0,
				]
	magics.sort()
	magics.reverse()
	magic = magics[-1]
	for n in magics:
		if n < abs(value):
			magic = n
			break
	return fround(value, magic)

class Box(PlotObject):
	"""Compound object, consists of 4 Axis, and a Grid

	Tickmarks are generated using xstart/ystart, and xinterval/yinterval.
	Between two tickmark Axis.xsubticks/ysubticks number of subticks are set.

	When the start and interval are not set, defaults are chosen, trying to
	set 4 labels at each axis. Using nice numbers that end at a 5 or 0.

	A grid is drawn if Axis.drawgrid is True, which is not the default.

	The Axis and Grid objects can be modified just like a normal Axis and Grid.
	But Axis.ticks sets the kind of ticks which are drawn e.g. if
	Box.leftticks is set to 'normal', tickmarks are generated each time
	it is plotted. If Box.leftticks is set to 'none', the ticks are removed.
	When set to 'log' the ticks are logarithmic distributed. 

	All axes are combined in the property Box.ticks, with the same properties
	as Box.###ticks.

	Default all axes have normal, inward tickmarks.

	Axes are labeled if set.
	The properties of the labeling are the same as the ticks e.g. if
	Box.bottomlabeling is set to 'normal', labels are generatedeach time it is
	plotted. If Box.bottomlabeling is set to 'none', the labels are removed.
	Again setting Box.bottomlabeling to 'log' generates logarithmic labeling.
	
	One can also use standard conversion strings like %1.2e. Axis willtry if the
	format is correct and apply it. If a wrong conversion string isgiven Axis
	will switch back to the default (%g).
	
	All axes are combined in the property Box.labeling, with the same properties
	as Box.###labeling.

	Default left and bottom axis have normal labels.

	The axes and the grid color properties are default set to None. And
	when the axes and the grid are plotted, the color of Box is used for default.
	
	So when the color of box is set to "red", each axis and the grid is drawn in
	red. When Box.grid.color is set to "blue", only the grid will be drawn in blue,
	and the rest in red. Set Box.grid.color to None to get the color of box again.

	Properties:
        - labeling
        - ticks
        - leftticks
        - bottomticks
        - topticks
        - rightticks
        - leftlabeling
        - bottomlabeling
        - toplabeling
        - rightlabeling 
        - xstart
        - ystart
        - xinterval
        - yinterval
        - xsubticks
        - ysubticks
        - leftaxis
        - rightaxis
        - topaxis
        - bottomaxis
	- drawgrid
        - grid
	- color
        - fontsize
        - matrix
	"""


	labeling = stringproperty()
	ticks = stringproperty()

	leftticks = stringproperty(default="normal")
	bottomticks = stringproperty(default="normal")
	topticks = stringproperty(default="normal")
	rightticks = stringproperty(default="normal")

	leftlabeling = stringproperty(default="normal")
	bottomlabeling = stringproperty(default="normal")
	toplabeling = stringproperty(default="none")
	rightlabeling = stringproperty(default="none")


	xstart = numberproperty(default=None)
	ystart = numberproperty(default=None)
	xinterval = numberproperty(default=None)
	yinterval = numberproperty(default=None)
	xsubticks = numberproperty(default=4)
	ysubticks = numberproperty(default=4)

	leftaxis = autoproperty()
	rightaxis = autoproperty()
	topaxis = autoproperty()
	bottomaxis = autoproperty()

	drawgrid = booleanproperty(default=0)
	grid = autoproperty()
	
	color = colorproperty()

	fontsize = fontsizeproperty()

	matrix = matrixproperty()




	def __init__(self, defaults=None, **properties):
		PlotObject.__init__(self, defaults=defaults, properties=properties)
		self.bottomaxis = Axis(side="bottom", color=None)
		self.bottomaxis.addChangeListener(self)
		self.topaxis = Axis(side="top", color=None)
		self.topaxis.addChangeListener(self)
		self.leftaxis = Axis(side="left", color=None)
		self.leftaxis.addChangeListener(self)
		self.rightaxis = Axis(side="right", color=None)
		self.rightaxis.addChangeListener(self)
		self.grid = Grid([], [], color=None)
		self.grid.addChangeListener(self)


	def plot(self, dev):
		if self.labeling != None:
			if self.leftlabeling.lower() != 'none':
				self.leftlabeling = self.labeling.lower()
			if self.rightlabeling.lower() != 'none':
				self.rightlabeling = self.labeling.lower()
			if self.toplabeling.lower() != 'none':
				self.toplabeling = self.labeling.lower()
			if self.bottomlabeling.lower() != 'none':
				self.bottomlabeling = self.labeling.lower()

		if self.ticks != None:
			if self.leftticks.lower() != 'none':
				self.leftticks = self.ticks.lower()
			if self.rightticks.lower() != 'none':
				self.rightticks = self.ticks.lower()
			if self.topticks.lower() != 'none':
				self.topticks = self.ticks.lower()
			if self.bottomticks.lower() != 'none':
				self.bottomticks = self.ticks.lower()

		if self.leftticks.lower() == 'none':
			self.leftaxis.ticklength = 0
			self.leftaxis.subticklength = 0
		if self.rightticks.lower() == 'none':
			self.rightaxis.ticklength = 0
			self.rightaxis.subticklength = 0
		if self.topticks.lower() == 'none':
			self.topaxis.ticklength = 0
			self.topaxis.subticklength = 0
		if self.bottomticks.lower() == 'none':
			self.bottomaxis.ticklength = 0
			self.bottomaxis.subticklength = 0

		if self.leftticks.lower() == 'log':
			self.yinterval = 1
			self.ysubticks = 9
		if self.rightticks.lower() == 'log':
			self.yinterval = 1
			self.ysubticks = 9
		if self.topticks.lower() == 'log':
			self.xinterval = 1
			self.xsubticks = 9
		if self.bottomticks.lower() == 'log':
			self.xinterval = 1
			self.xsubticks = 9



		dev.pushWorldMatrix(self.matrix)
		xticks, xsubticks = self.getXticks(dev)
		yticks, ysubticks = self.getYticks(dev)
		
		gridcolor = self.grid.color
		if self.grid.color == None:
			self.grid.color = self.color
		axes = [self.leftaxis, self.rightaxis, self.topaxis, self.bottomaxis]
		colors = [axis.color for axis in axes]
		
		for axis in axes:
			if axis.color == None:
				axis.color = self.color
				
					
		for axis,tick in [(self.leftaxis,self.leftticks),
				  (self.rightaxis,self.rightticks),
				  (self.bottomaxis,self.bottomticks),
				  (self.topaxis,self.topticks)]:
			if tick == None:
				axis.tickstyle = self.ticks
			else:
				axis.tickstyle = tick

		for axis, tick in [(self.leftaxis, self.leftticks), (self.rightaxis, self.rightticks)]:
			axis.tickstyle = tick
			axis.ticks = yticks
			axis.subticks = ysubticks
			axis.fontsize = self.fontsize


		for axis, tick in [(self.bottomaxis, self.bottomticks), (self.topaxis, self.topticks)]:
			axis.tickstyle = tick
			axis.ticks = xticks
			axis.subticks = xsubticks
			axis.fontsize = self.fontsize


		self.grid.xticks = xticks
		self.grid.yticks = yticks


		for axis, label in [(self.leftaxis, self.leftlabeling), (self.rightaxis, self.rightlabeling)]:
			if label.lower() != "none":
				axis.labelstyle = label
				axis.labels = label
				axis.fontsize = self.fontsize

			else:
				axis.labels = []
				axis.fontsize = self.fontsize
								
		for axis, label in [(self.bottomaxis, self.bottomlabeling), (self.topaxis, self.toplabeling)]:
			if label.lower() != "none":
				axis.labelstyle = label
				axis.labels = label
				axis.fontsize = self.fontsize
			else:
				axis.labels = []
				axis.fontsize = self.fontsize

		
		if self.drawgrid:
			self.grid.plot(dev)
		for axis in axes:
			axis.plot(dev)

		self.grid.color = gridcolor
		for axis, color in zip(axes, colors):
			axis.color = color

		dev.popWorldMatrix()



	def getXticks(self, dev):
		matrix = dev.getWorldMatrix().inverse()
		v1 = (matrix * (0,0)).x
		v2 = (matrix * (1,0)).x
		#print "x %f %f" % (v1, v2)

		interval = self.xinterval
		if interval == None:
			diff = v2 - v1
			ticks = 3
			tick = diff / 4
			error = diff / 8.0
			interval = magicnr(tick, error)

		start = self.xstart
		if start == None:
			start = fround(v1, interval)

		ticks = arange(start-interval,v2+interval*1.5, interval)
		subticks = []

		if self.xsubticks > 0:
			subinterval = float(interval) / (1+self.xsubticks)
			length = len(ticks)
			for i in range(length-1):
				v1 = ticks[i]+subinterval
				v2 = ticks[i+1]
				subticks += arange(v1, v2-subinterval/2, subinterval)


		return ticks, subticks

	def getYticks(self, dev):
		matrix = dev.getWorldMatrix().inverse()
		v1 = (matrix * (0,0)).y
		v2 = (matrix * (0,1)).y
		#print "y %f %f" % (v1, v2)

		interval = self.yinterval
		if interval == None:
			diff = v2 - v1
			ticks = 3
			tick = diff / 4
			error = diff / 10.0
			interval = magicnr(tick, error)

		start = self.ystart
		if start == None:
			start = fround(v1, interval)

		ticks = arange(start-interval,v2+interval*1.5, interval)
		subticks = []

		if self.ysubticks > 0:
			subinterval = float(interval) / (1+self.ysubticks)
			length = len(ticks)
			for i in range(length-1):
				v1 = ticks[i]+subinterval
				v2 = ticks[i+1]
				subticks += arange(v1, v2-subinterval/2, subinterval)

		return ticks, subticks

	def example():
		import Kplot

		print "Box example"
		print
		print 'Box is a compound object, containing four Axes'
		print 'The four Axes are referred to as:'
		print '- bottomaxis'
		print '- topaxis'
		print '- leftaxis'
		print '- rightaxis'
		print 'These axes heve the same properties as Kplot.Axis.'
		print
		print 'Box also has some properties concerning all axes,'
		print 'such as labeling, ticks, color, etc. (see help)'
		print
		print 'An example which contains a lot of properties:'
		print "box = Kplot.Box()"
		print "box.toplabeling = 'log'"
		print "box.topticks = 'log'"
		print "box.rightlabeling = 'normal'	"	
		print "box.rightaxis.textangle = Kplot.toradians(30)"
		print "box.rightaxis.ticklength = +5"
		print "box.rightaxis.subticklength = +2"
		print "box.leftlabeling = 'none'"		
		print "box.leftticks = 'none'"			
		print "box.bottomaxis.color='red'"
		print "box.bottomaxis.linewidth=1"
		print "box.bottomaxis.labelcolor='blue'"
		print "box.grid.color = 'green'"
		print "box.drawgrid = True"
		print "dev = Kplot.createDevice()"
		print "panel = dev.createPanel()"
		print "panel.add(box)"
		print "panel.setWorld(p1=(0, 0), p2=(5, 10))"
		print "dev.output()"
		print "dev.close()"
		box = Kplot.Box() 
		box.toplabeling = 'log'
		box.topticks = 'log'
		box.rightlabeling = 'normal'		
		box.rightaxis.textangle = Kplot.toradians(30)
		box.rightaxis.ticklength = +5
		box.rightaxis.subticklength = +2
		box.leftlabeling = 'none'		
		box.leftticks = 'none'		
		box.bottomaxis.color='red'
		box.bottomaxis.linewidth=1
		box.bottomaxis.labelcolor='blue'
		box.grid.color = 'green'
		box.drawgrid = True
		dev = Kplot.createDevice()
		panel = dev.createPanel()
		panel.add(box)
		panel.setWorld(p1=(0, 0), p2=(5, 10))
		dev.output()
		dev.close()

	example = staticmethod(example)

	def test():
		import Kplot

		box = Kplot.Box()
		box.rightlabeling = 'normal'
		box.toplabeling = 'normal'
		box.rightaxis.textangle = Kplot.toradians(22)
		box.leftaxis.textangle = Kplot.toradians(-90)
		box.topaxis.textangle = Kplot.toradians(80)
		box.bottomaxis.textangle = Kplot.toradians(-30)
		box.bottomaxis.valign = "top"
		box.bottomaxis.halign = "left"
		box.grid.color = "blue"
		box.leftaxis.ticklength = -5
		box.leftaxis.subticklength = -2
		box.leftaxis.labeloffset = 1

		dev = Kplot.createDevice()
		panel = dev.createPanel()
		panel.add(box)
		panel.setWorld(p1=(0, 0), p2=(130000, 9.9))
		dev.output()
		dev.close()

	test = staticmethod(test)



