"""Some objects that do not correspond to Device calls, but are rather primitive"""
from plotobject import PlotObject
from properties import *
from autoproperty import *
import types
from Kplot import Vector
from Numeric import log10,floor

def errorPositions(pos,error,type,minPos):
	if type =='log':
		if pos <= error[0]:
			e1 = minPos-.1
		else:
			e1 = log10(10**pos - 10**error[0])
		e2 = log10(10**pos + 10**error[1])
	else:
		e1 = pos - error[0]
		e2 = pos + error[1]
	return e1,e2

class Line(PlotObject):
	"""Draws a single line from one point to another.
	Input is two tuples.

	properties:
	- color
	- linestyle
	- linewidth
	- units
	"""
	color = colorproperty()
	linestyle = linestyleproperty()
	linewidth = linewidthproperty()
	p1 = pointproperty()
	p2 = pointproperty()
	matrix = matrixproperty()
	units = stringproperty()

	def __init__(self, p1, p2, defaults=None, **properties):
		"""Contructor

		Arguments:
		p1 -- the start point of the line
		p2 -- the end point of the line
		"""
		PlotObject.__init__(self, defaults=defaults, properties=properties)
		self.p1 = p1
		self.p2 = p2

	def plot(self, dev):
		dev.pushWorldMatrix(self.matrix)
		(x1, y1) = self.p1
		(x2, y2) = self.p2
		x = Numeric.array([x1, x2])
		y = Numeric.array([y1, y2])
		dev.plotPolyline(x, y, color=self.color, linestyle=self.linestyle, linewidth=self.linewidth, units=self.units)
		dev.popWorldMatrix()
	
	def example():
		import Kplot
		print "Line example"
		print
		print 'Line is a very simple plotobject.'
		print 'It just needs two points:'
		print 'line = Kplot.Line((-1, -1, (1, 2))'
		print 
		line = Kplot.Line((-1, -1), (1, 2))
		dev = Kplot.createDevice()
		panel = dev.createPanel()
		panel.add(line)
		panel.setWorld(p1=(-2, -2), p2=(3, 3))
		dev.output()
		raw_input(' Type <RETURN> for next page:')
		print
		print 'Line also has a few properties which can be altered'
		print 'for instance: line.linestyle="dotted"'
		print 'for more propeties see help(Line)' 
		line.linestyle="dotted"
		dev.output()
		dev.close()

	example = staticmethod(example)

class Marker(PlotObject):
	"""A single marker

	Several markertypes can be found in the Kplot.markertype package
	example:

		import Kplot

		marker1 = Kplot.Marker((10, 20), Kplot.markertype.tristar)
		marker2 = Kplot.Marker((10, 20), Kplot.markertype.x)

	The size of the marker is defined by the markersize combined with the units.
	The default units for a Marker object is points(1/72 inch), and the default 
	markersize is 6.
	a Marker can also have errors. Five atributes are created for that:
	xerror     - the error in the x direction. If this is an interger or float, the
	             errors are assumed to be symmetric. If xerror is a tuple or list,
		     the 1st element is the error in the negative x direction and the
		     2nd the error in the positive x direction.
	yerror     - same as xerror, but in the y direction.
	xerrortype - string indicating the kind of xerror. When set to 'log' the error
	             bars are plotted on a log scale. Default is 'normal'.
	yerrortype - same as xerrortype, but in the y direction.
	errortype  - sets both xerrortype and yerrortype
	barsize    - sets the size of the crossbar of the error. If not set, barsize = markersize

	properties:
	- location
	- markertype
	- color
	- markersize
	- linewidth
	- linestyle
	- fillstyle
	- xerror
	- yerror
	- barsize
	- errortype
	- xerrortype
	- yerrortype
	- units
	- matrix
	"""
	color = colorproperty()
	location = pointproperty()
	markersize = numberproperty(default=6) #autoproperty(typelist = [types.FloatType], default = 1)
	matrix = matrixproperty()
	markertype = autoproperty()
	units = stringproperty(default="points")
	linewidth = linewidthproperty()
	fillstyle = fillstyleproperty()
	# stuff for errorbars
	linestyle = linestyleproperty()
	xerror = autoproperty(typelist = [types.FloatType,types.IntType,types.ListType,types.TupleType])
	yerror = autoproperty(typelist = [types.FloatType,types.IntType,types.ListType,types.TupleType])
	barsize = numberproperty()
	errortype = stringproperty(default='normal')
	xerrortype = stringproperty()
	yerrortype = stringproperty()
	
	def __init__(self, location, markertype, defaults=None, **properties):
		"""Constructor

		Argument:
		location -- in the form of an (x, y) tuple
		markertype -- (an object with a plot method) see above or the Kplot.markertype 
					package.
		"""
		PlotObject.__init__(self, defaults=defaults, properties=properties)
		self.location = location
		self.markertype = markertype

	def plot(self, dev):
		if not self.barsize:
			self.barsize = self.markersize
		if not self.xerrortype:
			self.xerrortype = self.errortype
		if not self.yerrortype:
			self.yerrortype = self.errortype
		dev.pushWorldMatrix(self.matrix)
		self.markertype.plot(dev, self.location, self.color, self.linewidth, self.units, self.markersize, self.fillstyle, self.linestyle)
		scale = dev.getUnitMatrix(self.units) * dev.getViewportMatrix().inverse() * dev.getWorldMatrix().inverse()
		xmin = dev.getWorldMatrix().inverse()[0][-1]
		ymin = dev.getWorldMatrix().inverse()[1][-1]
		scale[0,2] = 0 
		scale[1,2] = 0
		sizeVec = Vector(0,0)
		sizeVec.x = .5 * self.barsize
		sizeVec.y = .5 * self.barsize
		sizeVec = scale * sizeVec
		if self.xerror:
			self.xerror = list(Numeric.array([self.xerror]).flat)
			if len(self.xerror) == 1:
				self.xerror.append(self.xerror[0])
			errorXl,errorXr = errorPositions(self.location[0],self.xerror,self.xerrortype,xmin)
			x = Numeric.array([errorXl,errorXr])
			y = Numeric.array([self.location[1],self.location[1]])
			xl = Numeric.array([errorXl,errorXl])
			xr = Numeric.array([errorXr,errorXr])
			yb = Numeric.array([self.location[1]-sizeVec[1],self.location[1]+sizeVec[1]])
			dev.plotPolyline(x , y,color=self.color,linestyle=self.linestyle,linewidth=self.linewidth,units=self.units)
			dev.plotPolyline(xl,yb,color=self.color,linestyle=self.linestyle,linewidth=self.linewidth,units=self.units)
			dev.plotPolyline(xr,yb,color=self.color,linestyle=self.linestyle,linewidth=self.linewidth,units=self.units)

		if self.yerror:
			self.yerror = list(Numeric.array([self.yerror]).flat)
			if len(self.yerror) == 1:
				self.yerror.append(self.yerror[0])
			errorYb,errorYt = errorPositions(self.location[1],self.yerror,self.yerrortype,ymin)
			x = Numeric.array([self.location[0],self.location[0]])
			y = Numeric.array([errorYb,errorYt])
			yb = Numeric.array([errorYb,errorYb])
			yt = Numeric.array([errorYt,errorYt])
			xb = Numeric.array([self.location[0]-sizeVec[0],self.location[0]+sizeVec[0]])
			dev.plotPolyline(x , y,color=self.color,linestyle=self.linestyle,linewidth=self.linewidth,units=self.units)
			dev.plotPolyline(xb,yb,color=self.color,linestyle=self.linestyle,linewidth=self.linewidth,units=self.units)
			dev.plotPolyline(xb,yt,color=self.color,linestyle=self.linestyle,linewidth=self.linewidth,units=self.units)
		dev.popWorldMatrix()

	def example():
		import Kplot

		print "Marker example"
		print
		print 'Marker is a plotobject which plots indivudual data points.'
		print 'The input is a position (tuple) and a markertype.'
		print 'for instance: marker = Kplot.Marker((2, 2), Kplot.markertype.x)'
		print 'which results in:'
		print 
		marker = Kplot.Marker((2, 2), Kplot.markertype.x)
		dev = Kplot.createDevice()
		panel = dev.createPanel()
		panel.add(marker)
		panel.setWorld(p1=(0, 0), p2=(4, 4))
		dev.output()
		raw_input(' Type <RETURN> for next page:')
		print
		print 'Ofcourse a lot of parameters can be changed (see help):'
		print 'marker = Kplot.Marker((2, 2), Kplot.markertype.x, units="cm", markersize=4, color="red")'
		print
		marker.units="cm"
		marker.markersize=4
		marker.color='red'
		dev.output()
		dev.close()
		
	example = staticmethod(example)

	def test():
		import Kplot

		print "marker should be 4 cm"
		marker = Kplot.Marker((2, 2), Kplot.markertype.circle, units="cm", markersize=4)

		dev = Kplot.createDevice()
		panel = dev.createPanel()
		panel.add(marker)
		panel.setWorld(p1=(0, 0), p2=(4, 4))
		dev.output()
		dev.close()

	test = staticmethod(test)


class Markers(PlotObject):
	"""Draws the same marker at multiple locations

	Needs two arrays for the positions: one array with x coordinates and one with y

	See Marker for more information
	"""
	color = colorproperty()
	x = arrayproperty()
	y = arrayproperty()
	markersize = numberproperty(default=8) #autoproperty(typelist = [types.FloatType], default = 1)
	matrix = matrixproperty()
	markertype = autoproperty()
	units = stringproperty(default="points")
	linewidth = linewidthproperty()
	fillstyle = fillstyleproperty()
	# stuff for errorbars
	linestyle = linestyleproperty()
	xerror = arrayproperty()
	yerror = arrayproperty()
	barsize = numberproperty()
	errortype = stringproperty(default='normal')
	xerrortype = stringproperty()
	yerrortype = stringproperty()

	def __init__(self, x, y, markertype, defaults=None, **properties):
		"""Constructor

		Arguments:
		x -- a sequence with all x elements
		y -- Idem, but for y
		markertype -- see Marker
		"""
		PlotObject.__init__(self, defaults=defaults, properties=properties)
		self.x = x
		self.y = y
		self.markertype = markertype

	def plot(self, dev):
		if not self.barsize:
			self.barsize = self.markersize
		if not self.xerrortype:
			self.xerrortype = self.errortype
		if not self.yerrortype:
			self.yerrortype = self.errortype
		dev.pushWorldMatrix(self.matrix)
		
		scale = dev.getUnitMatrix(self.units) * dev.getViewportMatrix().inverse() * dev.getWorldMatrix().inverse()
		xmin = dev.getWorldMatrix().inverse()[0][-1]
		ymin = dev.getWorldMatrix().inverse()[1][-1]
		scale[0,2] = 0
		scale[1,2] = 0
		sizeVec = Vector(0,0)
		sizeVec.x = .5 * self.barsize
		sizeVec.y = .5 * self.barsize
		sizeVec = scale * sizeVec

		length = min(len(self.x), len(self.y))
		if self.xerror:
			if len(Numeric.array(self.xerror).shape) == 1:
				self.xerror = Numeric.array([self.xerror,self.xerror])
			self.xerror = Numeric.transpose(self.xerror)
		if self.yerror:
			if len(Numeric.array(self.yerror).shape) == 1:
				self.yerror = Numeric.array([self.yerror,self.yerror])
			self.yerror = Numeric.transpose(self.yerror)

		for i in range(length):
			location = self.x[i], self.y[i]
			self.markertype.plot(dev, location, self.color, self.linewidth, self.units, self.markersize, self.fillstyle, self.linestyle)
			if self.xerror:
				errorXl,errorXr = errorPositions(location[0],self.xerror[i],self.xerrortype,xmin)
				x = Numeric.array([errorXl,errorXr])
				y = Numeric.array([location[1],location[1]])
				xl = Numeric.array([errorXl,errorXl])
				xr = Numeric.array([errorXr,errorXr])
				yb = Numeric.array([location[1]-sizeVec[1],location[1]+sizeVec[1]])
				dev.plotPolyline(x , y,color=self.color,linestyle=self.linestyle,linewidth=self.linewidth,units=self.units)
				dev.plotPolyline(xl,yb,color=self.color,linestyle=self.linestyle,linewidth=self.linewidth,units=self.units)
				dev.plotPolyline(xr,yb,color=self.color,linestyle=self.linestyle,linewidth=self.linewidth,units=self.units)

			if self.yerror:
				errorYb,errorYt = errorPositions(location[1],self.yerror[i],self.yerrortype,ymin)
				x = Numeric.array([location[0],location[0]])
				y = Numeric.array([errorYb,errorYt])
				yb = Numeric.array([errorYb,errorYb])
				yt = Numeric.array([errorYt,errorYt])
				xb = Numeric.array([location[0]-sizeVec[0],location[0]+sizeVec[0]])
				dev.plotPolyline(x , y,color=self.color,linestyle=self.linestyle,linewidth=self.linewidth,units=self.units)
				dev.plotPolyline(xb,yb,color=self.color,linestyle=self.linestyle,linewidth=self.linewidth,units=self.units)
				dev.plotPolyline(xb,yt,color=self.color,linestyle=self.linestyle,linewidth=self.linewidth,units=self.units)
		dev.popWorldMatrix()

	def example():
		import Kplot

		print "Markers example"
		print
		print 'Markers is very similar to Marker.'
		print 'The difference is that Markers needs two array as input.'
		print 'for instance:'
		print 'x = [0, 1, 2, 3, 4]'
		print 'y = [4, 1, 0, 3, 2]'
		print 'markers = Kplot.Markers(x, y, Kplot.markertype.tristar)'
		x = [0, 1, 2, 3, 4]
		y = [4, 1, 0, 3, 2]
		markers = Kplot.Markers(x, y, Kplot.markertype.tristar)
		dev = Kplot.createDevice()
		panel = dev.createPanel()
		panel.add(markers)
		panel.setWorld(p1=(-1, -1), p2=(5, 5))
		dev.output()
		dev.close()
		print 'For more information, see Marker'
		print

	example = staticmethod(example)


class Circle(PlotObject):
	"""A circle in world coordinates
	It can be streched when the aspect of the viewport-worldcoordinates is not
	square.

	Input is a position (tuple) and a radius

	properties:
	- radius
	- center
	- color
	- linestyle 
	- linewidth 
	- fillstyle 
	- matrix
	- units
	"""
	radius = autoproperty(typelist = [NumberType])
	center = pointproperty()
	color = colorproperty()
	linestyle = linestyleproperty()
	linewidth = linewidthproperty()
	fillstyle = fillstyleproperty()
	matrix = matrixproperty()
	units = stringproperty()

	def __init__(self, center, radius, defaults=None, **properties):
		"""Constructor
		Arguments:
		location -- center point of the circle
		radius -- radius of the circle in world coordinates
		"""
		PlotObject.__init__(self, defaults=defaults, properties=properties)
		self.center = center
		self.radius = radius

	def plot(self, dev):
		dev.pushWorldMatrix(self.matrix)
		#x, y = self.center
		#p1 = x - self.radius, y - self.radius
		#p2 = x + self.radius, y + self.radius
		dev.plotCircle(self.center, radius=self.radius, fillstyle=self.fillstyle,
			linestyle=self.linestyle, linewidth=self.linewidth,
			color=self.color, units=self.units)
		dev.popWorldMatrix()

	def example():
		import Kplot

		print "Circle example"
		print
		print 'The input of Circle is a position (tuple) and a radius (number):'
		print 'circle = Kplot.Circle((5, 5), 4)'
		circle = Kplot.Circle((5, 5), 4)
		dev = Kplot.createDevice()
		panel = dev.createPanel()
		panel.add(circle)
		panel.setWorld(p1=(0, 0), p2=(10, 10))
		dev.output()
		raw_input(' Type <RETURN> for next page:')
		print
		print 'Since Circle is similar to a Polygon, properties like linewidth, color,'
		print 'fillstyle, etc can be changed:'
		print 'circle = Kplot.Circle((5, 5), 4, fillstyle="hatched", linestyle="normal",'
		print 'color="slateblue")'
		print 'see help for more properties'
		circle.fillstyle="hatched"
		circle.linestyle="normal"
		circle.color="slateblue"
		dev.output()
		dev.close()
		print

	def test():
		import Kplot

		print "There should be 4 circles"
		print "   ", "hatched with normal linestyle"
		print "   ", "cross hatched with no linestyle"
		print "   ", "red solid"
		print "   ", "not filled, dashed linestyle"
		circles = []
		circles.append(Kplot.Circle((5, 5), 4, fillstyle="hatched"))
		circles.append(Kplot.Circle((15, 5), 4, fillstyle="crosshatched", linestyle="noline"))
		circles.append(Kplot.Circle((5, 15), 4, color="red"))
		circles.append(Kplot.Circle((15, 15), 4, fillstyle="nofill", linestyle="dashed"))

		dev = Kplot.createDevice()
		panel = dev.createPanel()
		panel.add(circles)
		panel.setWorld(p1=(0, 0), p2=(20, 20))
		dev.output()
		dev.close()
		pass

	test = staticmethod(test)
	example = staticmethod(example)



class Rectangle(PlotObject):
	"""Draws a single rectangle

	Input are two points (tuples), the lower left (called p1) and the upper right point (p2)

	Properties:
	- p1
	- p2
	- color
	- linestyle
	- linewidth
	- fillstyle
	- matrix
	- units
	"""
	color = colorproperty()
	linestyle = linestyleproperty()
	linewidth = linewidthproperty()
	fillstyle = fillstyleproperty()
	p1 = pointproperty()
	p2 = pointproperty()
	matrix = matrixproperty()
	units = stringproperty()

	def __init__(self, p1, p2, defaults=None, **properties):
		"""Constructor

		Arguments:
		p1 -- the lower left corner of the rectangle
		p2 -- the upper right corner of the rectangle

		"""
		PlotObject.__init__(self, defaults=defaults, properties=properties)
		self.p1 = p1
		self.p2 = p2

	def plot(self, dev):
		dev.pushWorldMatrix(self.matrix)
		(x1, y1), (x2, y2) = (self.p1, self.p2)
		x = [x1, x2, x2, x1]
		y = [y1, y1, y2, y2]
		dev.plotPolygon(x, y, color=self.color, linestyle=self.linestyle, linewidth=self.linewidth, fillstyle=self.fillstyle, units=self.units)
		dev.popWorldMatrix()

	def example():
		import Kplot

		print "Rectangle example"
		print
		print 'Rectangle needs two tuples as input, '
		print 'the lower left point of the rectangle and the upper right one.'
		print 'rect = Kplot.Rectangle((2, 2), (13, 8))'
		rect = Kplot.Rectangle((2, 2), (13, 8))
		dev = Kplot.createDevice()
		panel = dev.createPanel()
		panel.add(rect)
		panel.setWorld(p1=(0, 0), p2=(15, 15))
		dev.output()
		raw_input(' Type <RETURN> for next page:')
		print
		print 'Since Rectangle is similar to a Polygon, properties like linewidth, color,'
		print 'fillstyle, etc can be changed:'
		print 'rect = Kplot.Rectangle((2, 2), (13,8), fillstyle="crosshatched", linewidth=5,'
		print 'color="green")'
		print 'see help for more properties'
		rect.fillstyle="crosshatched"
		rect.linewidth=5
		rect.color="green"
		dev.output()
		dev.close()
		print

	example = staticmethod(example)

	def test():
		import Kplot

		print "There should be 4 rectangles"
		rects = []
		rects.append(Kplot.Rectangle((1, 1), (9, 9), fillstyle="hatched"))
		rects.append(Kplot.Rectangle((19, 1), (11, 9), fillstyle="crosshatched"))
		rects.append(Kplot.Rectangle((19, 19), (11, 11), fillstyle="solid"))
		rects.append(Kplot.Rectangle((1, 19), (9, 11), fillstyle="nofill"))

		dev = Kplot.createDevice()
		panel = dev.createPanel()
		panel.add(rects)
		panel.setWorld(p1=(0, 0), p2=(20, 20))
		dev.output()
		dev.close()

	test = staticmethod(test)

